function [f, g] = fpsa(x,pars)
% Emre Mengi (Modified on September 4, 2015)
%
% auxiliary function (to be called by eigopt_constrained.m)
% this computes the smallest singular value function
% and its gradient required for the computation of the
% epsilon-pseudospectral abscissa
%
% pars.A - must contain the matrix whose epsilon-pseudospectral abscissa is sought
% pars.epsln - must contain the epsilon
%
% f,g - function and gradient values



A = pars.A;
AA = pars.AA;

[n,m] = size(A);

BM = AA - (x(1)+i*x(2))*A' -(x(1)-i*x(2))*A + (x(1)^2 + x(2)^2)*eye(n);


if (n ~= m)
	error('the input matrix must be square');
end


if (n >= 200)
%%%%%%%%%%%%
% Large scale, use eigs
%%%%%%%%%%%%
	[V,D] = eigs(BM,1,'SM');
		
	f = D - pars.epsln^2; 
	g = [ real(V'*(-A'-A+2*x(1)*eye(n))*V);
			real(V'*(-i*A'+i*A+2*x(2)*eye(n))*V)];
else
%%%%%%%%%%%%
% Not Large, use eig
%%%%%%%%%%%%
	[V,D] = eig(BM);

	D1 = diag(real(D));
	[f,ind] = min(D1);

	f = f - pars.epsln^2;								  

	g = [ real(V(:,ind)'*(-A'-A+2*x(1)*eye(n))*V(:,ind));
		 real(V(:,ind)'*(-i*A'+i*A+2*x(2)*eye(n))*V(:,ind))];
end

								  
return;